<?php

namespace Drupal\pwa_firebase\Controller;

use Drupal\Core\Cache\CacheableResponse;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Controller routines for help routes.
 */
class PWAController extends ControllerBase {

  /**
   * The constructor.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request parameter.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $fileUrlGenerator
   *   The file URL generator.
   */
  public function __construct(protected Connection $database, protected Request $request, protected FileUrlGeneratorInterface $fileUrlGenerator) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('request_stack')->getCurrentRequest(),
      $container->get('file_url_generator'),
    );
  }

  /**
   * Function to generate manifest.json.
   */
  public function manifest() {
    $config = $this->config('pwa_firebase.settings');
    $logo = theme_get_setting('logo.url');
    // Support file svg.
    $type = "image/svg+xml";
    $size = '36x36 48x48 57x57 60x60 72x72 76x76 96x96 114x114 120x120 128x128 144x144 152x152 180x180 192x192 256x256 384x384 512x512';
    $icons = [
      [
        "src" => $logo,
        "sizes" => $size,
        "type" => $type,
        "purpose" => "any maskable",
      ],
    ];
    $path_parts = pathinfo($logo);
    $ext = strtolower($path_parts['extension']);
    if ($ext != 'svg') {
      // Check file icons.json in theme.
      $iconsFile = DRUPAL_ROOT . $path_parts['dirname'] . '/icons.json';
      if (file_exists($iconsFile)) {
        $jsonString = file_get_contents($iconsFile);
        $icons = current(json_decode($jsonString, TRUE));
        foreach ($icons as &$icon) {
          $icon['src'] = $path_parts["dirname"] . '/' . $icon['src'];
        }
      }
      else {
        $typeId = exif_imagetype(DRUPAL_ROOT . $logo);
        $type = image_type_to_mime_type($typeId);
        $icons = [
          [
            "src" => $logo,
            "sizes" => $size,
            "type" => $type,
            "purpose" => "any maskable",
          ],
        ];
      }
    }
    $lang = $this->languageManager()->getDefaultLanguage()->getId();
    if (!empty($config->get('lang'))) {
      $lang = $config->get('lang');
    }
    $theme_color = $config->get('theme_color') ?? '#000000';
    $bg_color = $config->get('background_color') ?? '#ffffff';
    $systemSite = $this->config('system.site');
    $manifest = [
      "name" => $systemSite->get('name'),
      "icons" => $icons,
      "start_url" => !empty($config->get('start_url')) ? $config->get('start_url') : '/',
      "scope" => !empty($config->get('scope')) ? $config->get('scope') : '/',
      "display" => !empty($config->get('display')) ? $config->get('display') : 'standalone',
      "lang" => $lang,
      "gcm_sender_id" => $config->get('firebase_sender_id'),
      "theme_color" => '#' . str_replace('#', '', !empty($theme_color) ? $theme_color : 'default_theme_color'),
      "background_color" => '#' . str_replace('#', '', !empty($bg_color) ? $bg_color : 'default_background_color'),
    ];
    $short_name = $config->get('short_name') ?? $systemSite->get('slogan');
    if (!empty($short_name)) {
      $manifest["short_name"] = $short_name;
    }

    $response = new CacheableResponse(
      json_encode($manifest, JSON_UNESCAPED_SLASHES),
      200,
      ['Content-Type' => 'application/json']
    );
    $meta_data = $response->getCacheableMetadata();
    $meta_data->addCacheTags(['manifestjson']);
    $meta_data->addCacheContexts(['languages:language_interface']);
    return $response;

  }

  /**
   * Function to generate service worker.
   *
   * @return \Drupal\Core\Cache\CacheableResponse
   *   A notification for a user.
   */
  public function serviceworker($notificationTitle = "", $notificationBody = "") {

    if (empty($notificationTitle)) {
      $notificationTitle = $this->config('system.site')->get('name');
    }
    if (empty($notificationBody)) {
      $notificationBody = $this->t('You have a notification of');
    }
    $response = new CacheableResponse('', 200);
    $config = $this->config('pwa_firebase.settings');

    $firebase_version = $config->get('firebase_version', '10.9.0');

    $importScripts = [
      'importScripts("https://www.gstatic.com/firebasejs/' . $firebase_version . '/firebase-app-compat.js");',
      'importScripts("https://www.gstatic.com/firebasejs/' . $firebase_version . '/firebase-messaging-compat.js");',
      "\n",
    ];
    $firebase_messaging_sw = implode("\n", $importScripts);
    if (!empty($config->get('service_worker'))) {
      $firebase_messaging_sw .= $config->get('service_worker');
      $firebase_messaging_sw .= "\n";
    }
    $icon = $this->fileUrlGenerator
      ->generateAbsoluteString(theme_get_setting('logo.url'));

    // Initial Firebase app in the service worker by passing messagingSenderId.
    $project_id = trim($config->get('firebase_project_id'));
    $firebase_config = [
      'apiKey' => trim($config->get('firebase_apiKey_id')),
      'authDomain' => $project_id . '.firebaseapp.com',
      'projectId' => $project_id,
      'storageBucket' => $project_id . '.appspot.com',
      'databaseURL' => 'https://' . $project_id . '.firebaseio.com',
      'messagingSenderId' => trim($config->get('firebase_sender_id')),
      'appId' => trim($config->get('firebase_app_id')),
    ];
    if (!empty($config->get('firebase_measurement_id'))) {
      $firebase_config['measurementId'] = $config->get('firebase_measurement_id');
    }
    $firebase_messaging_sw .= 'firebase.initializeApp(' . json_encode($firebase_config, JSON_UNESCAPED_SLASHES) . ');';
    // Retrieve an instance of Firebase Messaging so that
    // it can handle background messages.
    $firebase_messaging_sw .= '
const messaging = firebase.messaging();
messaging.onBackgroundMessage(function(payload) {
  console.log("[firebase-messaging-sw.js] Received background message ",payload);
  const notificationTitle = "' . $notificationTitle . '";
  const notificationOptions = {
      body: "' . $notificationBody . '",
      icon: "' . $icon . '",
  };
  return self.registration.showNotification(
      notificationTitle,
      notificationOptions,
  );
});

self.addEventListener("push", function(event) {
  const payload = event.data.json();
  const title = payload.notification.title;
  const options = {
    body: payload.notification.body,
    icon: payload.notification.icon
  };

  event.waitUntil(
    self.registration.showNotification(title, options)
  );
});
';

    $response->setContent($firebase_messaging_sw);
    // $cache_metadata = CacheableMetadata::createFromRenderArray($build);
    // $response->addCacheableDependency($cache_metadata);
    $response->headers->set('Content-type', 'application/javascript');
    $response->headers->set('Service-Worker-Allowed', '/');

    return $response;
  }

  /**
   * Function receives the user token and save's it in the database.
   *
   * @param string $token
   *   Token for each user.
   *
   * @return bool
   *   Sent or not.
   *
   * @throws \Exception
   */
  public function tokenReceived(string $token = '') {
    if (empty($token)) {
      $token = $this->request->request->get('token');
    }
    if (empty($token)) {
      return new JsonResponse(FALSE);
    }
    $hasToken = $this->database->select('pwa_firebase', 'f')
      ->fields('f')
      ->condition('token', $token, '=')
      ->execute()
      ->fetchAssoc();
    $action = $this->request->request->get('action') ?? NULL;
    if (!$hasToken) {
      $device = 'pc';
      $iPod = strpos($_SERVER['HTTP_USER_AGENT'], "iPod");
      $iPhone = strpos($_SERVER['HTTP_USER_AGENT'], "iPhone");
      $iPad = strpos($_SERVER['HTTP_USER_AGENT'], "iPad");
      $android = strpos($_SERVER['HTTP_USER_AGENT'], "Android");
      if ($iPad || $iPhone || $iPod) {
        $device = 'ios';
      }
      else {
        if ($android) {
          $device = 'android';
        }
      }

      $this->database->insert('pwa_firebase')->fields([
        'token' => $token,
        'uid' => $this->currentUser()->id(),
        'device' => $device,
        'created' => date("Y-m-d H:i:s"),
      ])->execute();
      return new JsonResponse([
        'device' => $device,
        'uid' => $this->currentUser()->id(),
      ]);
    }
    elseif (!empty($action) && $action == 'delete') {
      $this->database->delete('pwa_firebase')
        ->condition('token', $token)
        ->execute();
      return new JsonResponse(FALSE);
    }
    elseif (empty($hasToken['uid'])) {
      // Update uid with token.
      $this->database->update('pwa_firebase')
        ->fields(['uid' => $this->currentUser()->id()])
        ->condition('token', $token)
        ->execute();
    }
    return new JsonResponse(FALSE);
  }

}
